@@ -66,6 +66,9 @@ def main(
     ),
     version: bool = typer.Option(
         None, "--version", "-v", help="Show version and exit"
+    ),
+    editor: bool = typer.Option(
+        False, "--editor", "-e", help="Enable editor mode for document editing"
     )
 ) -> None:
     if version:
@@ -104,6 +107,7 @@ def main(
     ctx.obj["api_key"] = key
     ctx.obj["jarvis_env"] = jarvis_env
     ctx.obj["jarvis_url"] = jarvis_url
+    ctx.obj["editor"] = editor
     asyncio.run(_run_interactive_mode(
         conversation_id,
         provider=provider,
@@ -112,7 +116,8 @@ def main(
         jarvis_url=jarvis_url,
         live_mode=live,
         theme=theme,
-        debug=debug
+        debug=debug,
+        editor_mode=editor
     ))
 
 
@@ -315,6 +320,86 @@ def config_set(
         raise typer.Exit(1)
 
 
+# Login command
+@app.command()
+def login(
+    local: bool = typer.Option(False, "-local", help="Login to local environment"),
+    beta: bool = typer.Option(False, "-beta", help="Login to beta environment"),
+    prod: bool = typer.Option(False, "-prod", help="Login to production environment"),
+) -> None:
+    """Authenticate with Jarvis using OAuth flow"""
+    from rich.console import Console
+    from .login import login as do_login, extract_user_info, LoginError
+
+    console = Console()
+    config_manager = get_config_manager()
+    config = config_manager.get()
+
+    # Determine environment
+    jarvis_env = None
+    jarvis_url = None
+
+    if local:
+        jarvis_env = "local"
+        jarvis_url = JARVIS_ENV_URLS["local"]
+    elif beta:
+        jarvis_env = "beta"
+        jarvis_url = JARVIS_ENV_URLS["beta"]
+    elif prod:
+        jarvis_env = "prod"
+        jarvis_url = JARVIS_ENV_URLS["prod"]
+
+    # Display environment info
+    if jarvis_env:
+        console.print(f"\n[cyan]Logging in to {jarvis_env} environment[/cyan]")
+        console.print(f"[dim]URL: {jarvis_url}[/dim]\n")
+        config.jarvis.api_base_url = jarvis_url
+    elif config.jarvis.api_base_url:
+        console.print(f"\n[cyan]Logging in to: {config.jarvis.api_base_url}[/cyan]\n")
+    else:
+        console.print("\n[cyan]Logging in to Jarvis[/cyan]\n")
+
+    try:
+        # Perform OAuth login
+        jwt_token, expires_at = asyncio.run(do_login(console=console))
+
+        # Extract user info from token
+        user_info = extract_user_info(jwt_token)
+
+        # Save token to config
+        config.jarvis.jwt_token = jwt_token
+        config.jarvis.token_expires_at = expires_at.isoformat()
+
+        # Also update user_id from token if available
+        if user_info.get("user_id"):
+            config.jarvis.user_id = user_info["user_id"]
+
+        config_manager.save(config)
+
+        # Display success message with user info
+        console.print("\n[green]✓ Login successful![/green]\n")
+
+        if user_info.get("name"):
+            console.print(f"[yellow]Name:[/yellow] {user_info['name']}")
+        if user_info.get("email"):
+            console.print(f"[yellow]Email:[/yellow] {user_info['email']}")
+        if user_info.get("user_id"):
+            console.print(f"[yellow]User ID:[/yellow] {user_info['user_id']}")
+
+        console.print(f"\n[dim]Token expires at: {expires_at.strftime('%Y-%m-%d %H:%M:%S %Z')}[/dim]")
+        console.print(f"[dim]Config saved to: {config_manager.config_path}[/dim]\n")
+
+    except LoginError as e:
+        console.print(f"\n[red]✗ Login failed:[/red] {e}\n")
+        raise typer.Exit(1)
+    except KeyboardInterrupt:
+        console.print("\n[yellow]Login cancelled by user[/yellow]\n")
+        raise typer.Exit(0)
+    except Exception as e:
+        console.print(f"\n[red]✗ Unexpected error:[/red] {e}\n")
+        raise typer.Exit(1)
+
+
 # History command
 @app.command()
 def history(
@@ -347,7 +432,8 @@ async def _run_interactive_mode(
     jarvis_url: Optional[str] = None,
     live_mode: bool = False,
     theme: str = "claude_dark",
-    debug: bool = False
+    debug: bool = False,
+    editor_mode: bool = False
 ) -> None:
     debug_logger = init_debug_logger(enabled=debug)
     if debug:
@@ -419,6 +505,65 @@ async def _run_interactive_mode(
     else:
         session = session_manager.new_session()
 
+    # Handle editor mode initialization
+    editor_doc_data = None
+    editor_doc_url = None
+    editor_doc_name = None
+    if editor_mode:
+        from rich.prompt import Prompt
+        from rich.panel import Panel
+        from .client import extract_doc_id_from_url, fetch_lark_document
+        import json
+
+        display.console.print(Panel.fit(
+            "[cyan]Editor Mode[/cyan]\n"
+            "Please provide a Lark document URL to edit.",
+            border_style="cyan"
+        ))
+
+        doc_url = Prompt.ask("Document URL").strip()
+        doc_id = extract_doc_id_from_url(doc_url)
+
+        if not doc_id:
+            display.print_error("Invalid Lark document URL")
+            raise typer.Exit(1)
+
+        display.console.print("[yellow]Loading document...[/yellow]")
+
+        try:
+            editor_doc_data = await fetch_lark_document(doc_id)
+            editor_doc_url = doc_url
+            # Extract document name from nodes (first page node)
+            nodes = editor_doc_data.get("nodes", [])
+            editor_doc_name = "Unknown Document"
+            if nodes and nodes[0].get("type") == "page":
+                editor_doc_name = nodes[0].get("title", "Unknown Document")
+
+            # Store editor mode info in session
+            session.editor_mode = True
+            session.document_url = editor_doc_url
+            session.document_name = editor_doc_name
+
+            # Display document status header
+            display.console.print(Panel(
+                f"[green]current_document:[/green] {editor_doc_name}\n"
+                f"[green]url:[/green] {editor_doc_url}\n"
+                f"[green]status:[/green] success loaded",
+                title="[bold cyan]Editor Mode[/bold cyan]",
+                border_style="green"
+            ))
+            display.console.print()
+        except Exception as e:
+            display.print_error(f"Failed to load document: {e}")
+            raise typer.Exit(1)
+
+    # Define token refresh callback
+    def on_token_refresh(new_token: str, new_expires_at: str):
+        """Callback to save refreshed token to config"""
+        config.jarvis.jwt_token = new_token
+        config.jarvis.token_expires_at = new_expires_at
+        config_manager.save(config)
+
     # Create the appropriate client
     if provider == APIProvider.JARVIS:
         client = create_client(
@@ -426,6 +571,9 @@ async def _run_interactive_mode(
             config={
                 "base_url": config.jarvis.api_base_url,
                 "user_id": config.jarvis.effective_user_id,
+                "jwt_token": config.jarvis.jwt_token,
+                "token_expires_at": config.jarvis.token_expires_at,
+                "on_token_refresh": on_token_refresh,
             }
         )
     elif provider == APIProvider.OPENAI:
@@ -501,19 +649,27 @@ async def _run_interactive_mode(
                 else:
                     display.print_error(f"Unknown command: {user_input}")
                     continue
-            session.add_message("user", user_input)
+            # Prepare user input - add document context for first message in editor mode
+            final_user_input = user_input
+            if editor_mode and editor_doc_data and len(session.get_messages()) == 0:
+                import json
+                doc_json = json.dumps({"code": 0, "message": "success", "data": editor_doc_data}, ensure_ascii=False)
+                final_user_input = f"USER_INPUT:\n```{user_input}```\n\n=====doc content====\n{doc_json}\n==================="
+
+            session.add_message("user", final_user_input)
             try:
                 # Track if this is a new conversation
                 is_continuing = session.conversation_id is not None
-                
+
                 # Show conversation status (for debugging)
                 if debug and is_continuing:
                     debug_logger.log_info(f"Continuing conversation: {session.conversation_id}")
-                    
+
                 display.start_streaming()
                 async for chunk in client.chat_completion_stream(
                     messages=session.get_messages(),
                     conversation_id=session.conversation_id,
+                    mode="editor" if editor_mode else None,
                 ):
                     content_delta = display.process_chunk(chunk)
                     if content_delta:
@@ -531,6 +687,51 @@ async def _run_interactive_mode(
                 display.end_streaming()
                 if display._current_content:
                     session.add_message("assistant", display._current_content)
+
+                # Handle followup questions if any
+                if display.has_pending_questions():
+                    questions = display.get_pending_questions()
+                    display.display_followup_questions(questions)
+
+                    # Collect answers for each question
+                    answers = []
+                    for idx, question in enumerate(questions, 1):
+                        if len(questions) == 1:
+                            prompt_text = "Your answer: "
+                        else:
+                            prompt_text = f"Answer {idx}/{len(questions)}: "
+
+                        try:
+                            answer = await asyncio.get_event_loop().run_in_executor(
+                                None,
+                                lambda text=prompt_text: prompt_session.prompt(text)
+                            )
+                            if not answer and question.default_value:
+                                answer = question.default_value
+                                display.print_info(f"Using default: {answer}")
+                            answers.append(answer)
+                        except (EOFError, KeyboardInterrupt):
+                            display.print_info("\nFollowup cancelled")
+                            break
+
+                    # If we got all answers, continue the conversation
+                    if len(answers) == len(questions):
+                        combined_answer = "\n".join(answers) if len(answers) > 1 else answers[0]
+                        session.add_message("user", combined_answer)
+
+                        # Send followup response
+                        display.start_streaming()
+                        async for chunk in client.chat_completion_stream(
+                            messages=session.get_messages(),
+                            conversation_id=session.conversation_id,
+                            mode="editor" if editor_mode else None,
+                        ):
+                            content_delta = display.process_chunk(chunk)
+                            if content_delta:
+                                display.update_streaming_content(content_delta)
+                        display.end_streaming()
+                        if display._current_content:
+                            session.add_message("assistant", display._current_content)
                 
                 # Save session and show conversation ID info after first message
                 session_manager.auto_save_current()
@@ -629,6 +830,13 @@ async def _run_one_shot(
     else:
         session = session_manager.new_session()
 
+    # Define token refresh callback
+    def on_token_refresh(new_token: str, new_expires_at: str):
+        """Callback to save refreshed token to config"""
+        config.jarvis.jwt_token = new_token
+        config.jarvis.token_expires_at = new_expires_at
+        config_manager.save(config)
+
     # Create the appropriate client
     if provider == APIProvider.JARVIS:
         client = create_client(
@@ -636,6 +844,9 @@ async def _run_one_shot(
             config={
                 "base_url": config.jarvis.api_base_url,
                 "user_id": config.jarvis.effective_user_id,
+                "jwt_token": config.jarvis.jwt_token,
+                "token_expires_at": config.jarvis.token_expires_at,
+                "on_token_refresh": on_token_refresh,
             }
         )
     elif provider == APIProvider.OPENAI:
@@ -678,6 +889,13 @@ async def _run_one_shot(
         display.end_streaming()
         if display._current_content:
             session.add_message("assistant", display._current_content)
+
+        # Handle followup questions if any (one-shot mode limitation: can't continue)
+        if display.has_pending_questions():
+            questions = display.get_pending_questions()
+            display.display_followup_questions(questions)
+            display.print_info("Note: Use interactive mode to answer followup questions.")
+
         session_manager.auto_save_current()
         if session.conversation_id:
             display.print_info(f"Conversation ID: {session.conversation_id}")
