@@ -3,7 +3,8 @@ from rich.panel import Panel
 from rich.markdown import Markdown
 from rich.text import Text
 from rich.live import Live
-from typing import Optional, Dict, Any
+from rich.spinner import Spinner
+from typing import Optional, Dict, Any, List, Tuple
 from datetime import datetime
 import re
 from .models import JarvisStep, ChatCompletionChunk
@@ -12,6 +13,275 @@ from .themes import get_theme, ColorTheme
 from .markdown_utils import markdown_to_rich_markup
 
 
+class FollowupQuestion:
+    """Represents a followup question from the AI"""
+    def __init__(self, question: str, default_value: Optional[str] = None):
+        self.question = question
+        self.default_value = default_value
+
+
+class XMLTagParser:
+    """Unified XML tag parser for filtering and processing LLM response tags"""
+
+    def __init__(self):
+        # Tags that are completely hidden (no output)
+        self._hidden_tags = {
+            'use_mcp_tool', 'mcp_tool', 'goal', 'context', 'tool_name', 'arguments',
+            'use_agent', 'agent_name',
+            'knowledge_search', 'query',
+            'document_reading', 'urls'
+        }
+
+        # Tags that need special formatting (collect content then format)
+        self._formatted_tags = {
+            'update_todo_list', 'update_todo', 'todos'
+        }
+
+        # Tags that preserve content (remove tags only)
+        self._content_only_tags = {
+            'attempt_completion', 'result', 'result_summary'
+        }
+
+        # Tags that apply style in real-time (streaming)
+        self._styled_tags = {
+            'thinking'
+        }
+
+        # Tags that collect for special handling
+        self._special_tags = {
+            'ask_followup_question'
+        }
+
+        # State tracking
+        self._buffer: str = ""
+        self._current_hidden_tag: Optional[str] = None
+        self._current_formatted_tag: Optional[str] = None
+        self._current_styled_tag: Optional[str] = None
+        self._current_special_tag: Optional[str] = None
+        self._formatted_content: str = ""
+        self._special_content: str = ""
+
+        # Collected special data
+        self._followup_questions: List[FollowupQuestion] = []
+
+    def reset(self):
+        """Reset parser state"""
+        self._buffer = ""
+        self._current_hidden_tag = None
+        self._current_formatted_tag = None
+        self._current_styled_tag = None
+        self._current_special_tag = None
+        self._formatted_content = ""
+        self._special_content = ""
+        self._followup_questions = []
+
+    def parse(self, content: str, format_callback=None) -> Tuple[str, List[FollowupQuestion]]:
+        """
+        Parse XML tags in streaming content.
+        Returns tuple of (filtered_content, followup_questions)
+        format_callback: Optional function to format collected content (e.g., todo lists)
+        """
+        if not content:
+            return content, []
+
+        # Add to buffer
+        self._buffer += content
+        result = []
+        i = 0
+
+        while i < len(self._buffer):
+            # Check if we're in a hidden block
+            if self._current_hidden_tag:
+                closing_tag = f'</{self._current_hidden_tag}>'
+                closing_pos = self._buffer.find(closing_tag, i)
+
+                if closing_pos != -1:
+                    i = closing_pos + len(closing_tag)
+                    self._current_hidden_tag = None
+                    continue
+                else:
+                    # Need more data
+                    self._buffer = self._buffer[i:]
+                    return ''.join(result), []
+
+            # Check if we're in a special block (e.g., ask_followup_question)
+            if self._current_special_tag:
+                closing_tag = f'</{self._current_special_tag}>'
+                closing_pos = self._buffer.find(closing_tag, i)
+
+                if closing_pos != -1:
+                    # Collect content
+                    self._special_content += self._buffer[i:closing_pos]
+
+                    # Process based on tag type
+                    if self._current_special_tag == 'ask_followup_question':
+                        questions = self._extract_followup_questions(self._special_content)
+                        self._followup_questions.extend(questions)
+
+                    # Reset state
+                    i = closing_pos + len(closing_tag)
+                    self._current_special_tag = None
+                    self._special_content = ""
+                    continue
+                else:
+                    # Keep accumulating content
+                    self._special_content += self._buffer[i:]
+                    self._buffer = ""
+                    return ''.join(result), []
+
+            # Check if we're in a formatted block (collect then format)
+            if self._current_formatted_tag:
+                closing_tag = f'</{self._current_formatted_tag}>'
+                closing_pos = self._buffer.find(closing_tag, i)
+
+                if closing_pos != -1:
+                    # Collect content
+                    self._formatted_content += self._buffer[i:closing_pos]
+
+                    # Format using callback if provided
+                    if format_callback:
+                        formatted = format_callback(self._current_formatted_tag, self._formatted_content)
+                        result.append(formatted)
+                    else:
+                        # Default: just output the content
+                        result.append(self._formatted_content)
+
+                    # Reset state
+                    i = closing_pos + len(closing_tag)
+                    self._current_formatted_tag = None
+                    self._formatted_content = ""
+                    continue
+                else:
+                    # Keep accumulating content
+                    self._formatted_content += self._buffer[i:]
+                    self._buffer = ""
+                    return ''.join(result), []
+
+            # Check if we're in a styled block (stream with style marker)
+            if self._current_styled_tag:
+                closing_tag = f'</{self._current_styled_tag}>'
+                closing_pos = self._buffer.find(closing_tag, i)
+
+                if closing_pos != -1:
+                    # Output remaining content with style marker
+                    content_chunk = self._buffer[i:closing_pos]
+                    if content_chunk:
+                        result.append(f"__STYLED__{self._current_styled_tag}__{content_chunk}")
+                    result.append(f"__END_STYLED__{self._current_styled_tag}__")
+
+                    i = closing_pos + len(closing_tag)
+                    self._current_styled_tag = None
+                    continue
+                else:
+                    # Output content with style marker and keep buffering
+                    content_chunk = self._buffer[i:]
+                    if content_chunk:
+                        result.append(f"__STYLED__{self._current_styled_tag}__{content_chunk}")
+                    self._buffer = ""
+                    return ''.join(result), []
+
+            # Bounds check
+            if i >= len(self._buffer):
+                break
+
+            # Check for opening tags
+            if self._buffer[i] == '<':
+                tag_end = self._buffer.find('>', i)
+                if tag_end == -1:
+                    # Incomplete tag, need more data
+                    self._buffer = self._buffer[i:]
+                    return ''.join(result), self._get_pending_questions()
+
+                tag_content = self._buffer[i+1:tag_end]
+
+                # Check if it's a closing tag
+                if tag_content.startswith('/'):
+                    tag_name = tag_content[1:].strip()
+                    # Skip closing tags for content_only_tags and styled_tags
+                    if tag_name in self._content_only_tags or tag_name in self._styled_tags:
+                        i = tag_end + 1
+                        continue
+                    i = tag_end + 1
+                    continue
+
+                # It's an opening tag
+                tag_name = tag_content.split()[0] if ' ' in tag_content else tag_content
+
+                # Check if it's a hidden tag
+                if tag_name in self._hidden_tags:
+                    self._current_hidden_tag = tag_name
+                    i = tag_end + 1
+                    continue
+
+                # Check if it's a special tag
+                if tag_name in self._special_tags:
+                    self._current_special_tag = tag_name
+                    self._special_content = ""
+                    i = tag_end + 1
+                    continue
+
+                # Check if it's a formatted tag
+                if tag_name in self._formatted_tags:
+                    self._current_formatted_tag = tag_name
+                    self._formatted_content = ""
+                    i = tag_end + 1
+                    continue
+
+                # Check if it's a styled tag (stream with style)
+                if tag_name in self._styled_tags:
+                    self._current_styled_tag = tag_name
+                    result.append(f"__START_STYLED__{tag_name}__")
+                    i = tag_end + 1
+                    continue
+
+                # Check if it's a content_only tag (remove tag, keep content)
+                if tag_name in self._content_only_tags:
+                    i = tag_end + 1
+                    continue
+
+                # Not a special tag, keep it
+                result.append(self._buffer[i])
+                i += 1
+            else:
+                # Regular content
+                result.append(self._buffer[i])
+                i += 1
+
+        # Clear buffer if we've processed everything
+        self._buffer = ""
+        return ''.join(result), self._get_pending_questions()
+
+    def _extract_followup_questions(self, content: str) -> List[FollowupQuestion]:
+        """Extract question and default_value tags from ask_followup_question content"""
+        questions = []
+
+        # Find all <question> tags
+        question_pattern = r'<question>(.*?)</question>'
+        question_matches = re.finditer(question_pattern, content, re.DOTALL)
+
+        # Find all <default_value> tags
+        default_pattern = r'<default_value>(.*?)</default_value>'
+        default_matches = list(re.finditer(default_pattern, content, re.DOTALL))
+
+        for idx, q_match in enumerate(question_matches):
+            question_text = q_match.group(1).strip()
+            default_value = None
+
+            # Try to find corresponding default value
+            if idx < len(default_matches):
+                default_value = default_matches[idx].group(1).strip()
+
+            questions.append(FollowupQuestion(question_text, default_value))
+
+        return questions
+
+    def _get_pending_questions(self) -> List[FollowupQuestion]:
+        """Get and clear pending followup questions"""
+        questions = self._followup_questions.copy()
+        self._followup_questions = []
+        return questions
+
+
 class DisplayManager:
     def __init__(self, config: Optional[DisplayConfig] = None):
         self.console = Console()
@@ -19,8 +289,8 @@ class DisplayManager:
         self._current_content = ""
         self._live: Optional[Live] = None
         self._spinner_tasks: Dict[str, Any] = {}
-        self._xml_buffer = ""
-        self._in_thinking_tag = False
+        self._xml_parser = XMLTagParser()
+        self._pending_questions: List[FollowupQuestion] = []
 
     def print_user_message(self, content: str) -> None:
         # Display user message with styling
@@ -95,24 +365,33 @@ class DisplayManager:
         elif step_type == "mcp_tool_executing":
             if self.config.show_tools:
                 tool_name = data.tool_name or "unknown"
-                self.print_info(f"· Executing: {tool_name}...")
+                self.console.print(f"[yellow]◐[/yellow] [cyan]Executing tool: {tool_name}...[/cyan]")
 
         elif step_type == "mcp_tool_complete":
             if self.config.show_tools:
                 tool_name = data.tools.get("tool_name") if data.tools else "unknown"
                 success = data.success
                 style = "green" if success else "red"
-                self.console.print(f"[{style}]· Tool completed: {tool_name}[/{style}]")
+                self.console.print(f"[{style}]·[/{style}] Tool completed: {tool_name}")
 
         # Knowledge search
         elif step_type == "knowledge_search_start":
+            # Don't display anything here, wait for executing event
+            pass
+
+        elif step_type == "knowledge_search_executing":
             queries = data.queries or []
-            self.print_info(f"· Searching knowledge base: {len(queries)} queries")
+            query_count = len(queries) if queries else 0
+            if query_count > 0:
+                self.console.print(f"[yellow]◐[/yellow] [cyan]Searching knowledge base: {query_count} queries[/cyan]")
+            else:
+                self.console.print(f"[yellow]◐[/yellow] [cyan]Searching knowledge base...[/cyan]")
 
         elif step_type == "knowledge_search_complete":
             doc_links = data.doc_links or []
             doc_count = len(doc_links)
-            self.print_info(f"· Found {doc_count} relevant documents")
+            if doc_count > 0:
+                self.console.print(f"[green]·[/green] Found {doc_count} relevant documents")
 
             if doc_links and self.config.show_knowledge_sources:
                 self.console.print()
@@ -130,8 +409,15 @@ class DisplayManager:
             doc_count = data.total_documents or 0
             self.print_info(f"· Reading {doc_count} documents...")
 
+        elif step_type == "document_reading_executing":
+            current_url = data.current_url or ""
+            if current_url:
+                self.console.print(f"[yellow]◐[/yellow] [cyan]Reading document...[/cyan]")
+            else:
+                self.console.print(f"[yellow]◐[/yellow] [cyan]Reading documents...[/cyan]")
+
         elif step_type == "document_reading_complete":
-            self.print_info("· Document reading completed")
+            self.console.print(f"[green]·[/green] Document reading completed")
 
         # Agent execution
         elif step_type == "agent_start":
@@ -152,69 +438,125 @@ class DisplayManager:
         elif step_type == "error":
             self.print_error(data.error_message or "Unknown error")
 
-    def _filter_xml_tags(self, content: str) -> str:
-        """Remove XML tags but keep content for immediate streaming"""
-        self._xml_buffer += content
-        result = []
-        i = 0
+    def _format_callback(self, tag_name: str, content: str) -> str:
+        """Format collected content based on tag type"""
+        if tag_name in ('update_todo_list', 'update_todo', 'todos'):
+            return self._format_todo_list(content)
+        return content
 
-        while i < len(self._xml_buffer):
-            # Check for opening thinking tag
-            if self._xml_buffer[i:i+10] == '<thinking>':
-                # Skip the opening tag
-                i += 10
-                self._in_thinking_tag = True
-                continue
+    def _format_todo_list(self, content: str) -> str:
+        """Format todo list with status icons"""
+        if not content.strip():
+            return ""
 
-            # Check for closing thinking tag
-            if self._xml_buffer[i:i+11] == '</thinking>':
-                # Skip the closing tag
-                i += 11
-                self._in_thinking_tag = False
-                continue
+        # Extract content from <todos> tag if present
+        todos_match = re.search(r'<todos>(.*?)</todos>', content, re.DOTALL)
+        if todos_match:
+            content = todos_match.group(1)
 
-            # Check if we might be at the start of a tag but need more data
-            if i + 11 >= len(self._xml_buffer) and self._xml_buffer[i] == '<':
-                # Might be an incomplete tag, buffer it
-                self._xml_buffer = self._xml_buffer[i:]
-                return ''.join(result)
-
-            # Regular content, add to result (including content inside thinking tags)
-            result.append(self._xml_buffer[i])
-            i += 1
-
-        # Clear buffer after processing
-        self._xml_buffer = ""
-        return ''.join(result)
+        lines = content.strip().split('\n')
+        formatted = []
+        for line in lines:
+            line = line.strip()
+            if line.startswith('[x]'):
+                formatted.append(f"[green]✓[/green] ~~{line[3:].strip()}~~")
+            elif line.startswith('[-]'):
+                formatted.append(f"[yellow]•[/yellow] {line[3:].strip()}")
+            elif line.startswith('[ ]'):
+                formatted.append(f"[dim]○[/dim] {line[3:].strip()}")
+            elif line:
+                formatted.append(line)
+        return '\n' + '\n'.join(formatted) + '\n' if formatted else ""
 
     def start_streaming(self) -> None:
         # Start streaming display (accumulates content)
         self._current_content = ""
-        self._xml_buffer = ""
-        self._in_thinking_tag = False
+        self._xml_parser.reset()
+        self._pending_questions = []
         self.console.print(Text("Jarvis:", style="bold green"))
 
     def update_streaming_content(self, delta_content: str) -> None:
-        # Filter out XML tags first
-        filtered_content = self._filter_xml_tags(delta_content)
+        # Parse XML tags and extract followup questions
+        filtered_content, questions = self._xml_parser.parse(delta_content, self._format_callback)
+
+        # Store any pending questions
+        if questions:
+            self._pending_questions.extend(questions)
 
         if not filtered_content:
             return
 
         self._current_content += filtered_content
-        if self.config.markdown:
-            formatted_content = markdown_to_rich_markup(filtered_content)
-            self.console.print(formatted_content, end="")
+
+        # Process style markers for thinking tags
+        if '__STYLED__' in filtered_content or '__START_STYLED__' in filtered_content:
+            parts = re.split(r'(__START_STYLED__\w+__|__STYLED__\w+__|__END_STYLED__\w+__)', filtered_content)
+            for part in parts:
+                if not part:
+                    continue
+                if part.startswith('__START_STYLED__') or part.startswith('__END_STYLED__'):
+                    continue
+                elif part.startswith('__STYLED__'):
+                    match = re.match(r'__STYLED__(\w+)__(.*)', part, re.DOTALL)
+                    if match:
+                        tag_name = match.group(1)
+                        content = match.group(2)
+                        if content and tag_name == 'thinking' and self.config.show_thinking:
+                            # Apply markdown formatting to thinking content
+                            if self.config.markdown:
+                                formatted_content = markdown_to_rich_markup(content)
+                                self.console.print(formatted_content, end="", style="bright_black italic")
+                            else:
+                                self.console.print(content, end="", style="bright_black italic")
+                else:
+                    if self.config.markdown:
+                        formatted_content = markdown_to_rich_markup(part)
+                        self.console.print(formatted_content, end="")
+                    else:
+                        self.console.print(part, end="", style="green")
         else:
-            self.console.print(filtered_content, end="", style="green")
+            if self.config.markdown:
+                formatted_content = markdown_to_rich_markup(filtered_content)
+                self.console.print(formatted_content, end="")
+            else:
+                self.console.print(filtered_content, end="", style="green")
+
         self.console.file.flush()
 
     def end_streaming(self) -> None:
         # End streaming display
         self.console.print("\n")
         self._current_content = ""
-        self._xml_buffer = ""
-        self._in_thinking_tag = False
+        self._xml_parser.reset()
+
+    def has_pending_questions(self) -> bool:
+        """Check if there are pending followup questions"""
+        return len(self._pending_questions) > 0
+
+    def get_pending_questions(self) -> List[FollowupQuestion]:
+        """Get and clear pending followup questions"""
+        questions = self._pending_questions.copy()
+        self._pending_questions = []
+        return questions
+
+    def display_followup_questions(self, questions: List[FollowupQuestion]) -> None:
+        """Display followup questions in a panel"""
+        self.console.print()
+        question_text = "[bold cyan]The assistant needs more information:[/bold cyan]\n\n"
+
+        for idx, q in enumerate(questions, 1):
+            question_text += f"[yellow]{idx}.[/yellow] {q.question}\n"
+            if q.default_value:
+                question_text += f"   [dim](Default: {q.default_value})[/dim]\n"
+
+        panel = Panel(
+            question_text.strip(),
+            title="[bold]Followup Questions[/bold]",
+            border_style="yellow",
+            expand=False
+        )
+        self.console.print(panel)
+        self.console.print()
 
     def process_chunk(self, chunk: ChatCompletionChunk) -> Optional[str]:
         # Process a single chunk and return content delta if any
@@ -301,54 +643,28 @@ class LiveWorkflowDisplay:
         self._active_tool_start: Optional[float] = None
         self._active_tool_name: Optional[str] = None
         self._tool_status_line: Optional[int] = None
-        
-        # XML tag patterns for filtering - state machine
-        # Tags that are completely hidden (no output)
-        self._hidden_tags = {
-            'use_mcp_tool', 'goal', 'context', 'tool_name', 'arguments',
-            'use_agent', 'agent_name',
-            'knowledge_search', 'query',
-            'document_reading', 'urls',
-            'ask_followup_question', 'question', 'default_value'
-        }
-        # Tags that need special formatting (collect content then format)
-        self._formatted_tags = {
-            'update_todo_list', 'update_todo'  # Format with status icons
-        }
-        # Tags that preserve content (remove tags only)
-        self._content_only_tags = {
-            'attempt_completion', 'result', 'result_summary'
-        }
-        # Tags that apply style in real-time (streaming)
-        self._styled_tags = {
-            'thinking'  # Apply dimmed style while streaming
-        }
-        self._current_hidden_tag: Optional[str] = None
-        self._current_formatted_tag: Optional[str] = None
-        self._current_styled_tag: Optional[str] = None
-        self._formatted_content: str = ""
-        self._buffer: str = ""  # Buffer for incomplete tags
+
+        # Use unified XML parser
+        self._xml_parser = XMLTagParser()
+        self._pending_questions: List[FollowupQuestion] = []
 
 
-    def _format_thinking(self, content: str) -> str:
-        """Format thinking content with dimmed style"""
-        if not content.strip():
-            return ""
-        lines = content.strip().split('\n')
-        formatted = [f"[{self.theme.thinking_text}]{line}[/{self.theme.thinking_text}]" for line in lines]
-        return '\n' + '\n'.join(formatted) + '\n'
-    
+    def _format_callback(self, tag_name: str, content: str) -> str:
+        """Format collected content based on tag type"""
+        if tag_name in ('update_todo_list', 'update_todo', 'todos'):
+            return self._format_todo_list(content)
+        return content
+
     def _format_todo_list(self, content: str) -> str:
-        """Format todo list with status icons"""
+        """Format todo list with status icons using theme colors"""
         if not content.strip():
             return ""
-        
+
         # Extract content from <todos> tag if present
-        import re
         todos_match = re.search(r'<todos>(.*?)</todos>', content, re.DOTALL)
         if todos_match:
             content = todos_match.group(1)
-        
+
         lines = content.strip().split('\n')
         formatted = []
         for line in lines:
@@ -363,182 +679,6 @@ class LiveWorkflowDisplay:
                 formatted.append(line)
         return '\n' + '\n'.join(formatted) + '\n' if formatted else ""
 
-    def _parse_xml_tags(self, content: str) -> str:
-        """Parse XML tags with state machine for streaming content"""
-        if not content:
-            return content
-        
-        # Add to buffer
-        self._buffer += content
-        result = []
-        i = 0
-        
-        while i < len(self._buffer):
-            # Check if we're in a hidden block
-            if self._current_hidden_tag:
-                closing_tag = f'</{self._current_hidden_tag}>'
-                closing_pos = self._buffer.find(closing_tag, i)
-                
-                if closing_pos != -1:
-                    i = closing_pos + len(closing_tag)
-                    self._current_hidden_tag = None
-                    continue
-                else:
-                    self._buffer = self._buffer[i:]
-                    return ''.join(result)
-            
-            # Check if we're in a formatted block (collect then format)
-            if self._current_formatted_tag:
-                closing_tag = f'</{self._current_formatted_tag}>'
-                closing_pos = self._buffer.find(closing_tag, i)
-                
-                if closing_pos != -1:
-                    # Collect content
-                    self._formatted_content += self._buffer[i:closing_pos]
-                    
-                    # Recursively parse content for nested tags
-                    parsed_content = self._parse_nested_tags(self._formatted_content)
-                    
-                    # Format based on tag type
-                    if self._current_formatted_tag in ('todos', 'update_todo', 'update_todo_list'):
-                        formatted = self._format_todo_list(parsed_content)
-                        result.append(formatted)
-                    
-                    # Reset state
-                    i = closing_pos + len(closing_tag)
-                    self._current_formatted_tag = None
-                    self._formatted_content = ""
-                    continue
-                else:
-                    # Keep accumulating content
-                    self._formatted_content += self._buffer[i:]
-                    self._buffer = ""
-                    return ''.join(result)
-            
-            # Check if we're in a styled block (stream with style marker)
-            if self._current_styled_tag:
-                closing_tag = f'</{self._current_styled_tag}>'
-                closing_pos = self._buffer.find(closing_tag, i)
-                
-                if closing_pos != -1:
-                    # Output remaining content with style marker
-                    content_chunk = self._buffer[i:closing_pos]
-                    if content_chunk:
-                        result.append(f"__STYLED__{self._current_styled_tag}__{content_chunk}")
-                    result.append(f"__END_STYLED__{self._current_styled_tag}__")
-                    
-                    i = closing_pos + len(closing_tag)
-                    self._current_styled_tag = None
-                    continue
-                else:
-                    # Output content with style marker and keep buffering
-                    content_chunk = self._buffer[i:]
-                    if content_chunk:
-                        result.append(f"__STYLED__{self._current_styled_tag}__{content_chunk}")
-                    self._buffer = ""
-                    return ''.join(result)
-            
-            # Bounds check
-            if i >= len(self._buffer):
-                break
-            
-            # Check for opening tags
-            if self._buffer[i] == '<':
-                tag_end = self._buffer.find('>', i)
-                if tag_end == -1:
-                    self._buffer = self._buffer[i:]
-                    return ''.join(result)
-                
-                tag_content = self._buffer[i+1:tag_end]
-                
-                # Check if it's a closing tag
-                if tag_content.startswith('/'):
-                    tag_name = tag_content[1:].strip()
-                    # Skip closing tags for content_only_tags and styled_tags
-                    if tag_name in self._content_only_tags or tag_name in self._styled_tags:
-                        i = tag_end + 1
-                        continue
-                    i = tag_end + 1
-                    continue
-                
-                # It's an opening tag
-                tag_name = tag_content.split()[0] if ' ' in tag_content else tag_content
-                
-                # Check if it's a hidden tag
-                if tag_name in self._hidden_tags:
-                    self._current_hidden_tag = tag_name
-                    i = tag_end + 1
-                    continue
-                
-                # Check if it's a formatted tag
-                if tag_name in self._formatted_tags:
-                    self._current_formatted_tag = tag_name
-                    self._formatted_content = ""
-                    i = tag_end + 1
-                    continue
-                
-                # Check if it's a styled tag (stream with style)
-                if tag_name in self._styled_tags:
-                    self._current_styled_tag = tag_name
-                    result.append(f"__START_STYLED__{tag_name}__")
-                    i = tag_end + 1
-                    continue
-                
-                # Check if it's a content_only tag (remove tag, keep content)
-                if tag_name in self._content_only_tags:
-                    i = tag_end + 1
-                    continue
-                
-                # Not a special tag, keep it
-                result.append(self._buffer[i])
-                i += 1
-            else:
-                # Regular content
-                result.append(self._buffer[i])
-                i += 1
-        
-        # Clear buffer if we've processed everything
-        self._buffer = ""
-        return ''.join(result)
-    
-    def _parse_nested_tags(self, content: str) -> str:
-        """Parse nested tags within formatted content"""
-        result = []
-        i = 0
-        
-        while i < len(content):
-            if content[i] == '<':
-                tag_end = content.find('>', i)
-                if tag_end == -1:
-                    result.append(content[i])
-                    i += 1
-                    continue
-                
-                tag_content = content[i+1:tag_end]
-                
-                # Check if it's a closing tag
-                if tag_content.startswith('/'):
-                    tag_name = tag_content[1:].strip()
-                    if tag_name in self._content_only_tags:
-                        i = tag_end + 1
-                        continue
-                
-                # It's an opening tag
-                tag_name = tag_content.split()[0] if ' ' in tag_content else tag_content
-                
-                # Remove content_only tags
-                if tag_name in self._content_only_tags:
-                    i = tag_end + 1
-                    continue
-                
-                result.append(content[i])
-                i += 1
-            else:
-                result.append(content[i])
-                i += 1
-        
-        return ''.join(result)
-
     def process_step(self, step: JarvisStep) -> None:
         """Process a Jarvis step with serial event stream display"""
         step_type = step.type
@@ -633,10 +773,20 @@ class LiveWorkflowDisplay:
 
         # Knowledge search
         elif step_type == "knowledge_search_start":
+            # Don't display anything here, wait for executing event
+            pass
+
+        elif step_type == "knowledge_search_executing":
             queries = data.queries or []
-            self.console.print(
-                f"[{self.theme.status_running}]·[/{self.theme.status_running}] [{self.theme.knowledge_text}]Searching knowledge base ({len(queries)} queries)...[/{self.theme.knowledge_text}]"
-            )
+            query_count = len(queries) if queries else 0
+            if query_count > 0:
+                self.console.print(
+                    f"[{self.theme.status_running}]◐[/{self.theme.status_running}] [{self.theme.knowledge_text}]Searching knowledge base: {query_count} queries[/{self.theme.knowledge_text}]"
+                )
+            else:
+                self.console.print(
+                    f"[{self.theme.status_running}]◐[/{self.theme.status_running}] [{self.theme.knowledge_text}]Searching knowledge base...[/{self.theme.knowledge_text}]"
+                )
 
         elif step_type == "knowledge_search_complete":
             doc_links = data.doc_links or []
@@ -663,6 +813,11 @@ class LiveWorkflowDisplay:
                 f"[{self.theme.status_running}]·[/{self.theme.status_running}] [{self.theme.knowledge_text}]Reading {doc_count} documents...[/{self.theme.knowledge_text}]"
             )
 
+        elif step_type == "document_reading_executing":
+            self.console.print(
+                f"[{self.theme.status_running}]◐[/{self.theme.status_running}] [{self.theme.knowledge_text}]Reading documents...[/{self.theme.knowledge_text}]"
+            )
+
         elif step_type == "document_reading_complete":
             self.console.print(f"[{self.theme.status_complete}]·[/{self.theme.status_complete}] Documents read")
 
@@ -714,28 +869,31 @@ class LiveWorkflowDisplay:
     def start_streaming(self) -> None:
         """Start streaming content display"""
         self._current_content = ""
+        self._xml_parser.reset()
+        self._pending_questions = []
         self.console.print()
         self.console.print(Text("Jarvis:", style=f"{self.theme.assistant_text} bold"))
 
     def update_streaming_content(self, delta_content: str) -> None:
         """Update streaming content - print immediately for real-time streaming"""
         self._current_content += delta_content
-        
-        # Parse and filter XML tags
-        filtered_content = self._parse_xml_tags(delta_content)
-        
+
+        # Parse and filter XML tags, extract followup questions
+        filtered_content, questions = self._xml_parser.parse(delta_content, self._format_callback)
+
+        # Store any pending questions
+        if questions:
+            self._pending_questions.extend(questions)
+
         # Only print if there's actual content after filtering
         if filtered_content:
-            # Process style markers
-            import re
-            
             # Split by style markers
             parts = re.split(r'(__START_STYLED__\w+__|__STYLED__\w+__|__END_STYLED__\w+__)', filtered_content)
-            
+
             for part in parts:
                 if not part:
                     continue
-                
+
                 # Check for style markers
                 if part.startswith('__START_STYLED__'):
                     continue  # Just marks the start, no output
@@ -748,12 +906,23 @@ class LiveWorkflowDisplay:
                         tag_name = match.group(1)
                         content = match.group(2)
                         if content:
-                            # Apply style based on tag
-                            if tag_name == 'thinking':
-                                self.console.print(content, end="", style=self.theme.thinking_text)
-                            else:
-                                self.console.print(content, end="", style=self.theme.assistant_text)
+                            # Apply style based on tag with markdown support
+                            if tag_name == 'thinking' and self.config.show_thinking:
+                                # Apply markdown formatting to thinking content
+                                if self.config.markdown:
+                                    formatted_content = markdown_to_rich_markup(content)
+                                    self.console.print(formatted_content, end="", style=self.theme.thinking_text)
+                                else:
+                                    self.console.print(content, end="", style=self.theme.thinking_text)
+                            elif tag_name != 'thinking':
+                                # Apply markdown formatting to other styled content
+                                if self.config.markdown:
+                                    formatted_content = markdown_to_rich_markup(content)
+                                    self.console.print(formatted_content, end="", style=self.theme.assistant_text)
+                                else:
+                                    self.console.print(content, end="", style=self.theme.assistant_text)
                 else:
+                    # Check if content already has Rich markup
                     if '[' in part and ']' in part and '/' in part:
                         self.console.print(part, end="")
                     elif part.strip() or part == '\n':
@@ -762,12 +931,42 @@ class LiveWorkflowDisplay:
                             self.console.print(formatted_content, end="")
                         else:
                             self.console.print(part, end="", style=self.theme.assistant_text)
-            
+
         self.console.file.flush()
 
     def end_streaming(self) -> None:
         """End streaming content display"""
         self.console.print("\n")
+        self._xml_parser.reset()
+
+    def has_pending_questions(self) -> bool:
+        """Check if there are pending followup questions"""
+        return len(self._pending_questions) > 0
+
+    def get_pending_questions(self) -> List[FollowupQuestion]:
+        """Get and clear pending followup questions"""
+        questions = self._pending_questions.copy()
+        self._pending_questions = []
+        return questions
+
+    def display_followup_questions(self, questions: List[FollowupQuestion]) -> None:
+        """Display followup questions in a themed panel"""
+        self.console.print()
+        question_text = f"[{self.theme.accent_primary} bold]The assistant needs more information:[/{self.theme.accent_primary} bold]\n\n"
+
+        for idx, q in enumerate(questions, 1):
+            question_text += f"[{self.theme.accent_secondary}]{idx}.[/{self.theme.accent_secondary}] {q.question}\n"
+            if q.default_value:
+                question_text += f"   [{self.theme.system_text}](Default: {q.default_value})[/{self.theme.system_text}]\n"
+
+        panel = Panel(
+            question_text.strip(),
+            title=f"[{self.theme.accent_primary} bold]Followup Questions[/{self.theme.accent_primary} bold]",
+            border_style=self.theme.border_main,
+            expand=False
+        )
+        self.console.print(panel)
+        self.console.print()
 
     def process_chunk(self, chunk: ChatCompletionChunk) -> Optional[str]:
         """Process a chunk and return content delta"""
